@@ -5,23 +5,28 @@ A Python library of interactive CLI elements you have been looking for
 
 __license__ = 'MIT'
 
+import math
 import warnings
-from ast import literal_eval
 from typing import Any, Callable, List, Optional, Tuple, Type, Union
 
 from rich.console import Console
 from rich.live import Live
 from yakh import get_key
-from yakh.key import Keys
+from yakh.key import Key, Keys
 
 from beaupy._internals import (
+    Abort,
     ConversionError,
+    TargetType,
     ValidationError,
     _cursor_hidden,
     _format_option_select,
+    _paginate_back,
+    _paginate_forward,
     _render_option_select_multiple,
     _render_prompt,
     _update_rendered,
+    _validate_prompt_value,
 )
 
 console = Console()
@@ -64,13 +69,55 @@ class Config:
 
     Attributes:
         raise_on_interrupt(bool): If True, functions will raise KeyboardInterrupt whenever one is encountered when waiting for input,
-        otherwise, they will return some sane alternative to their usual return (e.g.: None, [] ). Defaults to False.
+        otherwise, they will return some sane alternative to their usual return. For `select`, `prompt` and `confirm` this means `None`,
+        while for `select_multiple` it means an empty list - `[]`. Defaults to False.
+        raise_on_escape(bool): If True, functions will raise Abort whenever the escape key is encountered when waiting for input, otherwise,
+        they will return some sane alternative to their usual return. For `select`, `prompt` and `confirm` this means `None`, while for
+        `select_multiple` it means an empty list - `[]`.  Defaults to False.
     """
 
     raise_on_interrupt: bool = False
+    raise_on_escape: bool = False
 
 
-TargetType = Any
+_navigation_keys = [DefaultKeys.up, DefaultKeys.down, DefaultKeys.right, DefaultKeys.left, DefaultKeys.home, DefaultKeys.end]
+
+
+def _navigate_select(
+    index: int,
+    page: int,
+    keypress: Key,
+    total_options: int,
+    pagination: int,
+    total_pages: int,
+    page_size: int,
+    show_from: int,
+    show_to: int,
+) -> Tuple[int, int]:
+    if keypress in DefaultKeys.up:
+        if index <= show_from and pagination:
+            page = _paginate_back(page, total_pages)
+        index -= 1
+        index = index % total_options
+    elif keypress in DefaultKeys.down:
+        if index > show_to - 2 and pagination:
+            page = _paginate_forward(page, total_pages)
+        index += 1
+        index = index % total_options
+    elif keypress in DefaultKeys.right and pagination:
+        page = _paginate_forward(page, total_pages)
+        index = (page - 1) * page_size
+    elif keypress in DefaultKeys.left and pagination:
+        page = _paginate_back(page, total_pages)
+        index = (page - 1) * page_size
+    elif keypress in DefaultKeys.home:
+        page = 1
+        index = 0
+    elif keypress in DefaultKeys.end:
+        page = total_pages
+        index = total_options - 1
+
+    return index, page
 
 
 def prompt(
@@ -118,24 +165,21 @@ def prompt(
                     raise KeyboardInterrupt()
                 return None
             elif keypress in DefaultKeys.confirm:
-                str_value = ''.join(value)
                 try:
-                    if target_type is bool:
-                        result: bool = literal_eval(str_value)
-                        if not isinstance(result, bool):
-                            raise ValueError()
-                    else:
-                        result: target_type = target_type(str_value)  # type: ignore
-                    if validator(result):
-                        return result
-                    else:
-                        error = f"Input {'<secure_input>' if secure else '`'+str_value+'`'} is invalid"
-                        if raise_validation_fail:
-                            raise ValidationError(error)
-                except ValueError:
-                    error = f"Input {'<secure_input>' if secure else '`'+str_value+'`'} cannot be converted to type `{target_type}`"
+                    return _validate_prompt_value(
+                        value=value,
+                        target_type=target_type,
+                        validator=validator,
+                        secure=secure,
+                    )
+                except ValidationError as e:
+                    if raise_validation_fail:
+                        raise e
+                    error = str(e)
+                except ConversionError as e:
                     if raise_type_conversion_fail:
-                        raise ConversionError(error) from None
+                        raise e
+                    error = str(e)
             elif keypress in DefaultKeys.backspace:
                 if cursor_index > 0:
                     cursor_index -= 1
@@ -146,8 +190,6 @@ def prompt(
             elif keypress in DefaultKeys.right:
                 if cursor_index < len(value):
                     cursor_index += 1
-            elif keypress in DefaultKeys.escape:
-                return None
             elif keypress in DefaultKeys.up + DefaultKeys.down:
                 pass
             elif keypress in DefaultKeys.home:
@@ -157,14 +199,15 @@ def prompt(
             elif keypress in DefaultKeys.delete:
                 if cursor_index < len(value):
                     del value[cursor_index]
+            elif keypress in DefaultKeys.escape:
+                if Config.raise_on_escape:
+                    raise Abort(keypress)
+                return None
             else:
                 value.insert(cursor_index, str(keypress))
                 cursor_index += 1
 
 
-Selection = Union[int, Any]
-
-
 def select(
     options: List[Union[Tuple[int, ...], str]],
     preprocessor: Callable[[Any], Any] = lambda val: val,
@@ -173,7 +216,9 @@ def select(
     cursor_index: int = 0,
     return_index: bool = False,
     strict: bool = False,
-) -> Union[Selection, None]:
+    pagination: bool = False,
+    page_size: int = 5,
+) -> Union[int, Any, None]:
     """A prompt that allows selecting one option from a list of options
 
     Args:
@@ -188,6 +233,8 @@ def select(
         return_index (bool, optional): If `True`, `select` will return the index of selected element in options. Defaults to `False`.
         strict (bool, optional): If empty `options` is provided and strict is `False`, None will be returned,
         if it's `True`, `ValueError` will be thrown. Defaults to False.
+        pagination (bool, optional): If `True`, pagination will be used. Defaults to False.
+        page_size (int, optional): Number of options to show on a single page if pagination is enabled. Defaults to 5.
 
     Raises:
         ValueError: Thrown if no `options` are povided and strict is `True`
@@ -207,17 +254,22 @@ def select(
             cursor_style = 'white'
 
         index: int = cursor_index
+        page: int = index // page_size + 1
+        total_pages = math.ceil(len(options) / page_size)
 
         while True:
-            rendered = (
+            show_from = (page - 1) * page_size
+            show_to = min(show_from + page_size, len(options))
+            rendered = (  # noqa: ECE001
                 '\n'.join(
                     [
                         _format_option_select(
-                            i=i, cursor_index=index, option=preprocessor(option), cursor_style=cursor_style, cursor=cursor
+                            i=i, cursor_index=index % page_size, option=preprocessor(option), cursor_style=cursor_style, cursor=cursor
                         )
-                        for i, option in enumerate(options)
+                        for i, option in enumerate(options[show_from:show_to] if pagination else options)
                     ]
                 )
+                + (f'[grey58]\n\nPage {page}/{total_pages}[/grey58]' if pagination and total_pages > 1 else '')  # noqa: W503
                 + '\n\n(Confirm with [bold]enter[/bold])'  # noqa: W503
             )
             _update_rendered(live, rendered)
@@ -226,23 +278,18 @@ def select(
                 if Config.raise_on_interrupt:
                     raise KeyboardInterrupt()
                 return None
-            elif keypress in DefaultKeys.up:
-                if index > 0:
-                    index -= 1
-            elif keypress in DefaultKeys.down:
-                if index < len(options) - 1:
-                    index += 1
+            elif any([keypress in navigation_keys for navigation_keys in _navigation_keys]):
+                index, page = _navigate_select(index, page, keypress, len(options), pagination, total_pages, page_size, show_from, show_to)
             elif keypress in DefaultKeys.confirm:
                 if return_index:
                     return index
                 return options[index]
             elif keypress in DefaultKeys.escape:
+                if Config.raise_on_escape:
+                    raise Abort(keypress)
                 return None
 
 
-Selections = List[Selection]
-
-
 def select_multiple(
     options: List[Union[Tuple[int, ...], str]],
     preprocessor: Callable[[Any], Any] = lambda val: val,
@@ -255,7 +302,9 @@ def select_multiple(
     maximal_count: Optional[int] = None,
     return_indices: bool = False,
     strict: bool = False,
-) -> Selections:
+    pagination: bool = False,
+    page_size: int = 5,
+) -> List[Union[int, Any]]:
     """A prompt that allows selecting multiple options from a list of options
 
     Args:
@@ -275,6 +324,8 @@ def select_multiple(
                                          of ticked elements in options. Defaults to `False`.
         strict (bool, optional): If empty `options` is provided and strict is `False`, None will be returned,
                                  if it's `True`, `ValueError` will be thrown. Defaults to False.
+        pagination (bool, optional): If `True`, pagination will be used. Defaults to False.
+        page_size (int, optional): Number of options to show on a single page if pagination is enabled. Defaults to 5.
 
     Raises:
         KeyboardInterrupt: Raised when keyboard interrupt is encountered and Config.raise_on_interrupt is True
@@ -298,24 +349,28 @@ def select_multiple(
             ticked_indices = []
 
         index = cursor_index
+        page: int = index // page_size + 1
+        total_pages = math.ceil(len(options) / page_size)
 
-        max_index = len(options) - (1 if True else 0)
         error_message = ''
         while True:
-            rendered = (
+            show_from = (page - 1) * page_size
+            show_to = min(show_from + page_size, len(options))
+            rendered = (  # noqa: ECE001
                 '\n'.join(
                     [
                         _render_option_select_multiple(
                             option=preprocessor(option),
-                            ticked=i in ticked_indices,
+                            ticked=i + show_from in ticked_indices,
                             tick_character=tick_character,
                             tick_style=tick_style,
-                            selected=i == index,
+                            selected=i == index % page_size,
                             cursor_style=cursor_style,
                         )
-                        for i, option in enumerate(options)
+                        for i, option in enumerate(options[show_from:show_to] if pagination else options)
                     ]
                 )
+                + (f'[grey58]\n\nPage {page}/{total_pages}[/grey58]' if pagination and total_pages > 1 else '')  # noqa: W503
                 + '\n\n(Mark with [bold]space[/bold], confirm with [bold]enter[/bold])'  # noqa: W503
             )
             if error_message:
@@ -327,12 +382,8 @@ def select_multiple(
                 if Config.raise_on_interrupt:
                     raise KeyboardInterrupt()
                 return []
-            elif keypress in DefaultKeys.up:
-                if index > 0:
-                    index -= 1
-            elif keypress in DefaultKeys.down:
-                if index + 1 <= max_index:
-                    index += 1
+            elif any([keypress in navigation_keys for navigation_keys in _navigation_keys]):
+                index, page = _navigate_select(index, page, keypress, len(options), pagination, total_pages, page_size, show_from, show_to)
             elif keypress in DefaultKeys.select:
                 if index in ticked_indices:
                     ticked_indices.remove(index)
@@ -349,6 +400,8 @@ def select_multiple(
                 else:
                     break
             elif keypress in DefaultKeys.escape:
+                if Config.raise_on_escape:
+                    raise Abort(keypress)
                 return []
         if return_indices:
             return ticked_indices
@@ -417,14 +470,16 @@ def confirm(
             elif keypress in DefaultKeys.backspace:
                 if current_message:
                     current_message = current_message[:-1]
-            elif keypress in DefaultKeys.escape:
-                return None
             elif keypress in DefaultKeys.confirm:
                 if is_selected:
                     break
             elif keypress in DefaultKeys.tab:
                 if is_selected:
                     current_message = yes_text if is_yes else no_text
+            elif keypress in DefaultKeys.escape:
+                if Config.raise_on_escape:
+                    raise Abort(keypress)
+                return None
             else:
                 current_message += str(keypress)
                 match_yes = yes_text
